"use strict";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var __decorate = this && this.__decorate || function (decorators, target, key, desc) {
  var c = arguments.length,
      r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc,
      d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};

var __metadata = this && this.__metadata || function (k, v) {
  if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};

var __param = this && this.__param || function (paramIndex, decorator) {
  return function (target, key) {
    decorator(target, key, paramIndex);
  };
};

var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AppBuilder = void 0;
/*!
 * Copyright (c) Microsoft. All rights reserved.
 * Licensed under the MIT license. See LICENSE file in the project.
 */

const apollo_server_core_1 = require("apollo-server-core");

const apollo_server_fastify_1 = require("apollo-server-fastify");

const fastify_1 = __importDefault(require("fastify"));

const graphql_1 = require("graphql");

const tsyringe_1 = require("tsyringe");

const injectors_1 = require("../injectors");

let AppBuilder = class AppBuilder {
  constructor(_appContext, _logger, _schema, _requestContextProviders) {
    this._appContext = _appContext;
    this._logger = _logger;
    this._schema = _schema;
    this._requestContextProviders = _requestContextProviders;
    this._apolloServer = this.createApolloServer();
  }

  get config() {
    return this._appContext.configuration;
  }

  createApolloServer() {
    return new apollo_server_fastify_1.ApolloServer({
      schema: this._schema,
      plugins: this.config.serverPlayground ? undefined : [apollo_server_core_1.ApolloServerPluginLandingPageDisabled],
      logger: this._logger,
      introspection: this.config.serverIntrospection,
      context: async ({
        request
      }) => {
        const ctx = _objectSpread(_objectSpread({}, this._appContext), {}, {
          request: {}
        });

        for (const rcp of this._requestContextProviders) {
          const result = await rcp.apply(ctx, request);
          ctx.request = _objectSpread(_objectSpread({}, ctx.request), result);
        }

        return ctx;
      },
      debug: false
    });
  }

  async start(preRun) {
    const app = (0, fastify_1.default)({
      logger: this._logger
    });
    await this._apolloServer.start();
    app.setErrorHandler(async (error, request, reply) => {
      if (error.validation) {
        reply.code(400).send({
          message: error.message,
          validation: error.validation
        });
      } else {
        reply.code(error.statusCode || 500).send({
          messsage: error.message
        });
      }
    });
    /* eslint-disable  @essex/adjacent-await */

    await (preRun === null || preRun === void 0 ? void 0 : preRun(app));
    await app.register(this._apolloServer.createHandler({
      cors: true
    }));
    await app.listen({
      port: this.config.serverPort
    });
  }

};
AppBuilder = __decorate([(0, tsyringe_1.singleton)(), __param(0, (0, tsyringe_1.inject)(injectors_1.BaseInjectorNames.AppContext)), __param(1, (0, tsyringe_1.inject)(injectors_1.BaseInjectorNames.Logger)), __param(2, (0, tsyringe_1.inject)(injectors_1.BaseInjectorNames.Schema)), __param(3, (0, tsyringe_1.inject)(injectors_1.BaseInjectorNames.RequestContextProviders)), __metadata("design:paramtypes", [Object, Object, graphql_1.GraphQLSchema, Array])], AppBuilder);
exports.AppBuilder = AppBuilder;